@@ -43,6 +43,7 @@ static void usage(const char* exe) {
 					"  -x <scids>   ID of the service component to be played (requires service ID)\n"
 					"  -g <gain>    USB stick gain to pass to DAB live source (auto gain is default)\n"
 					"  -r <path>    Path for recordings (default: %s)\n"
+					"  -P <size>    Recording prebuffer size in seconds (default: %zu)\n"
 					"  -p           Output PCM to stdout instead of using SDL\n"
 					"  -u           Output untouched audio stream to stdout instead of using SDL\n"
 					"  -S           Initially disable slideshow\n"
@@ -50,7 +51,8 @@ static void usage(const char* exe) {
 					"  file         Input file to be played (stdin, if not specified)\n",
 					DABLiveETISource::TYPE_DAB2ETI.c_str(),
 					DABLiveETISource::TYPE_ETI_CMDLINE.c_str(),
-					options_default.recordings_path.c_str()
+					options_default.recordings_path.c_str(),
+					options_default.rec_prebuffer_size_s
 			);
 	exit(1);
 }
@@ -72,7 +74,7 @@ int main(int argc, char **argv) {
 
 	// option args
 	int c;
-	while((c = getopt(argc, argv, "hd:D:C:c:l:g:r:s:x:puSL")) != -1) {
+	while((c = getopt(argc, argv, "hd:D:C:c:l:g:r:P:s:x:puSL")) != -1) {
 		switch(c) {
 		case 'h':
 			usage(argv[0]);
@@ -106,6 +108,9 @@ int main(int argc, char **argv) {
 		case 'r':
 			options.recordings_path = optarg;
 			break;
+		case 'P':
+			options.rec_prebuffer_size_s = strtol(optarg, nullptr, 0);
+			break;
 		case 'p':
 			options.pcm_output = true;
 			break;
@@ -203,6 +208,7 @@ DABlinGTK::DABlinGTK(DABlinGTKOptions options) {
 
 	initial_channel_appended = false;
 	rec_file = nullptr;
+	rec_duration_ms = 0;
 
 	slideshow_window.set_transient_for(*this);
 
@@ -381,14 +387,14 @@ void DABlinGTK::InitWidgets() {
 void DABlinGTK::AddChannels() {
 	if(options.displayed_channels.empty()) {
 		// add all channels
-		for(dab_channels_t::const_iterator it = dab_channels.cbegin(); it != dab_channels.cend(); it++)
-			AddChannel(it, options.gain);
+		for(const dab_channels_t::value_type& dab_channel : dab_channels)
+			AddChannel(dab_channel, options.gain);
 	} else {
 		// add specific channels
 		string_vector_t channels = MiscTools::SplitString(options.displayed_channels, ',');
-		for(string_vector_t::const_iterator ch_it = channels.cbegin(); ch_it != channels.cend(); ch_it++) {
+		for(const std::string& channel : channels) {
 			int gain = options.gain;
-			string_vector_t parts = MiscTools::SplitString(*ch_it, ':');
+			string_vector_t parts = MiscTools::SplitString(channel, ':');
 			switch(parts.size()) {
 			case 2:
 				gain = strtol(parts[1].c_str(), nullptr, 0);
@@ -396,25 +402,25 @@ void DABlinGTK::AddChannels() {
 			case 1: {
 				dab_channels_t::const_iterator it = dab_channels.find(parts[0]);
 				if(it != dab_channels.cend())
-					AddChannel(it, gain);
+					AddChannel(*it, gain);
 				else
 					fprintf(stderr, "DABlinGTK: The channel '%s' is not supported; ignoring!\n", parts[0].c_str());
 				break; }
 			default:
-				fprintf(stderr, "DABlinGTK: The format of channel '%s' is not supported; ignoring!\n", ch_it->c_str());
+				fprintf(stderr, "DABlinGTK: The format of channel '%s' is not supported; ignoring!\n", channel.c_str());
 				continue;
 			}
 		}
 	}
 }
 
-void DABlinGTK::AddChannel(dab_channels_t::const_iterator &it, int gain) {
+void DABlinGTK::AddChannel(const dab_channels_t::value_type& dab_channel, int gain) {
 	Gtk::ListStore::iterator row_it = combo_channels_liststore->append();
 	Gtk::TreeModel::Row row = *row_it;
-	row[combo_channels_cols.col_string] = it->first;
-	row[combo_channels_cols.col_channel] = DAB_LIVE_SOURCE_CHANNEL(it->first, it->second, gain);
+	row[combo_channels_cols.col_string] = dab_channel.first;
+	row[combo_channels_cols.col_channel] = DAB_LIVE_SOURCE_CHANNEL(dab_channel.first, dab_channel.second, gain);
 
-	if(it->first == options.initial_channel)
+	if(dab_channel.first == options.initial_channel)
 		initial_channel_it = row_it;
 }
 
@@ -423,11 +429,14 @@ void DABlinGTK::SetService(const LISTED_SERVICE& service) {
 	if(!service.IsNone()) {
 		char sid_string[7];
 		snprintf(sid_string, sizeof(sid_string), "0x%04X", service.sid);
-		Glib::ustring label = FICDecoder::ConvertLabelToUTF8(service.label);
+
+		std::string charset_name;
+		Glib::ustring label = FICDecoder::ConvertLabelToUTF8(service.label, &charset_name);
 
 		set_title(label + " - DABlin");
 		frame_combo_services.set_tooltip_text(
 				"Short label: \"" + DeriveShortLabel(label, service.label.short_label_mask) + "\"\n"
+				"Label charset: " + charset_name + "\n"
 				"SId: " + sid_string + (!service.IsPrimary() ? " (SCIdS: " + std::to_string(service.scids) + ")" : "") + "\n"
 				"SubChId: " + std::to_string(service.audio_service.subchid) + "\n"
 				"Audio type: " + (service.audio_service.dab_plus ? "DAB+" : "DAB")
@@ -459,6 +468,8 @@ void DABlinGTK::SetService(const LISTED_SERVICE& service) {
 	// if the audio service changed, reset format/DL/slide + switch
 	if(!eti_player->IsSameAudioService(service.audio_service)) {
 		// stop playback of old service
+		if(options.rec_prebuffer_size_s > 0)
+			eti_player->RemoveUntouchedStreamConsumer(this);
 		eti_player->SetAudioService(AUDIO_SERVICE());
 
 		pad_decoder->Reset();
@@ -474,8 +485,18 @@ void DABlinGTK::SetService(const LISTED_SERVICE& service) {
 			slideshow_window.ClearSlide();
 		}
 
+		if(options.rec_prebuffer_size_s > 0) {
+			std::lock_guard<std::mutex> lock(rec_mutex);
+
+			// clear prebuffer
+			rec_prebuffer.clear();
+			rec_prebuffer_filled_ms = 0;
+		}
+
 		// start playback of new service
 		eti_player->SetAudioService(service.audio_service);
+		if(options.rec_prebuffer_size_s > 0)
+			eti_player->AddUntouchedStreamConsumer(this);
 	}
 
 	if(service.HasSLS()) {
@@ -506,29 +527,44 @@ void DABlinGTK::on_tglbtn_record() {
 			Gtk::TreeModel::Row row = *row_it;
 			service = row[combo_services_cols.col_service];
 		}
-		std::string label = FICDecoder::ConvertLabelToUTF8(service.label);
+		std::string label = FICDecoder::ConvertLabelToUTF8(service.label, nullptr);
 
 		// escape forbidden '/' character
 		std::string label_cleaned = label;
-		for(size_t i = 0; i < label_cleaned.length(); i++)
-			if(label_cleaned[i] == '/')
-				label_cleaned[i] = '_';
+		for(char& c : label_cleaned)
+			if(c == '/')
+				c = '_';
 
-		std::string rec_filename = options.recordings_path + "/" + std::string(now_string) + " - " + label_cleaned + "." + eti_player->GetUntouchedStreamFileExtension();
-		FILE* new_rec_file = fopen(rec_filename.c_str(), "wb");
+		std::string new_rec_filename = options.recordings_path + "/" + std::string(now_string) + " - " + label_cleaned + "." + eti_player->GetUntouchedStreamFileExtension();
+		FILE* new_rec_file = fopen(new_rec_filename.c_str(), "wb");
 		if(new_rec_file) {
 			// disable channel/service switch
 			combo_channels.set_sensitive(false);	// parent frame already non-sensitive, if channels not available
 			combo_services.set_sensitive(false);
 
-			fprintf(stderr, "DABlinGTK: recording started into '%s'\n", rec_filename.c_str());
+			fprintf(stderr, "DABlinGTK: recording started into '%s'\n", new_rec_filename.c_str());
 
 			{
-				std::lock_guard<std::mutex> lock(rec_file_mutex);
+				std::lock_guard<std::mutex> lock(rec_mutex);
+
 				rec_file = new_rec_file;
+				rec_filename = new_rec_filename;
+
+				// write prebuffer
+				for(const RecSample& s : rec_prebuffer)
+					if(fwrite(&s.data[0], s.data.size(), 1, rec_file) != 1)
+						perror("DABlinGTK: error while writing untouched stream prebuffer to file");
+				rec_duration_ms = rec_prebuffer_filled_ms;
+
+				// clear prebuffer
+				rec_prebuffer.clear();
+				rec_prebuffer_filled_ms = 0;
+
+				UpdateRecStatus();
 			}
 
-			eti_player->AddUntouchedStreamConsumer(this);
+			if(options.rec_prebuffer_size_s == 0)
+				eti_player->AddUntouchedStreamConsumer(this);
 		} else {
 			int fopen_errno = errno;	// save before overwrite
 			tglbtn_record.set_active(false);
@@ -539,16 +575,18 @@ void DABlinGTK::on_tglbtn_record() {
 			hint.run();
 		}
 	} else {
-		std::lock_guard<std::mutex> lock(rec_file_mutex);
+		std::lock_guard<std::mutex> lock(rec_mutex);
 
 		// only stop recording, if active (handles error on recording start)
 		if(rec_file) {
-			eti_player->RemoveUntouchedStreamConsumer(this);
+			if(options.rec_prebuffer_size_s == 0)
+				eti_player->RemoveUntouchedStreamConsumer(this);
 
 			fclose(rec_file);
 			rec_file = nullptr;
 
 			fprintf(stderr, "DABlinGTK: recording stopped\n");
+			UpdateRecStatus();
 
 			// enable channel/service switch
 			combo_channels.set_sensitive(true);		// parent frame already non-sensitive, if channels not available
@@ -557,15 +595,62 @@ void DABlinGTK::on_tglbtn_record() {
 	}
 }
 
-void DABlinGTK::ProcessUntouchedStream(const uint8_t* data, size_t len) {
-	std::lock_guard<std::mutex> lock(rec_file_mutex);
+void DABlinGTK::ProcessUntouchedStream(const uint8_t* data, size_t len, size_t duration_ms) {
+	std::lock_guard<std::mutex> lock(rec_mutex);
 
+	// if recording in progress
 	if(rec_file) {
+		long int rec_duration_ms_old = rec_duration_ms;
+
+		// write sample
 		if(fwrite(data, len, 1, rec_file) != 1)
-			perror("DABlinGTK: error while writing untouched stream to file");
+			perror("DABlinGTK: error while writing untouched stream sample to file");
+		rec_duration_ms += duration_ms;
+
+		// update status only on seconds change
+		if(rec_duration_ms / 1000 != rec_duration_ms_old / 1000)
+			UpdateRecStatus();
+	} else {
+		// if prebuffer enabled
+		if(options.rec_prebuffer_size_s > 0) {
+			long int rec_prebuffer_ms_old = rec_prebuffer_filled_ms;
+
+			// append sample
+			rec_prebuffer.emplace_back(data, len, duration_ms);
+			rec_prebuffer_filled_ms += duration_ms;
+
+			// remove samples while needed
+			while(rec_prebuffer_filled_ms - rec_prebuffer.front().duration_ms >= options.rec_prebuffer_size_s * 1000) {
+				rec_prebuffer_filled_ms -= rec_prebuffer.front().duration_ms;
+				rec_prebuffer.pop_front();
+			}
+
+			// update status only on seconds change
+			if(rec_prebuffer_filled_ms / 1000 != rec_prebuffer_ms_old / 1000)
+				UpdateRecStatus();
+		}
 	}
 }
 
+void DABlinGTK::UpdateRecStatus() {
+	// mutex must already be locked!
+
+	std::string tooltip_text;
+
+	// if recording in progress
+	if(rec_file) {
+		tooltip_text =
+				"File name: \"" + rec_filename + "\"\n"
+				"Duration: " + MiscTools::MsToTimecode(rec_duration_ms);
+	} else {
+		// if prebuffer enabled
+		if(rec_prebuffer_filled_ms > 0)
+			tooltip_text = "Prebuffer: " + MiscTools::MsToTimecode(rec_prebuffer_filled_ms) + " / " + MiscTools::MsToTimecode(options.rec_prebuffer_size_s * 1000);
+	}
+
+	tglbtn_record.set_tooltip_text(tooltip_text);
+}
+
 
 void DABlinGTK::on_tglbtn_mute() {
 	eti_player->SetAudioMute(tglbtn_mute.get_active());
@@ -700,7 +785,7 @@ void DABlinGTK::ETIUpdateProgressEmitted() {
 void DABlinGTK::ETIChangeFormatEmitted() {
 //	fprintf(stderr, "### ETIChangeFormatEmitted\n");
 
-	label_format.set_label(eti_change_format.Pop());
+	label_format.set_label(eti_change_format.Pop().GetSummary());
 }
 
 void DABlinGTK::FICChangeEnsembleEmitted() {
@@ -711,10 +796,13 @@ void DABlinGTK::FICChangeEnsembleEmitted() {
 	char eid_string[7];
 	snprintf(eid_string, sizeof(eid_string), "0x%04X", new_ensemble.eid);
 
-	Glib::ustring label = FICDecoder::ConvertLabelToUTF8(new_ensemble.label);
+	std::string charset_name;
+	Glib::ustring label = FICDecoder::ConvertLabelToUTF8(new_ensemble.label, &charset_name);
+
 	label_ensemble.set_label(label);
 	frame_label_ensemble.set_tooltip_text(
 			"Short label: \"" + DeriveShortLabel(label, new_ensemble.label.short_label_mask) + "\"\n"
+			"Label charset: " + charset_name + "\n"
 			"EId: " + eid_string);
 }
 
@@ -723,7 +811,7 @@ void DABlinGTK::FICChangeServiceEmitted() {
 
 	LISTED_SERVICE new_service = fic_change_service.Pop();
 
-	std::string label = FICDecoder::ConvertLabelToUTF8(new_service.label);
+	std::string label = FICDecoder::ConvertLabelToUTF8(new_service.label, nullptr);
 	Glib::ustring combo_label = label;
 	if(new_service.multi_comps)
 		combo_label = (!new_service.IsPrimary() ? "» " : "") + combo_label + (new_service.IsPrimary() ? " »" : "");
@@ -757,6 +845,10 @@ void DABlinGTK::FICChangeServiceEmitted() {
 	}
 }
 
+void DABlinGTK::FICDiscardedFIB() {
+	fprintf(stderr, "\x1B[33m" "(FIB)" "\x1B[0m" " ");
+}
+
 void DABlinGTK::on_combo_channels() {
 	Gtk::TreeModel::Row row = *combo_channels.get_active();
 	DAB_LIVE_SOURCE_CHANNEL channel = row[combo_channels_cols.col_channel];
