@@ -20,6 +20,7 @@
 #define DABLIN_GTK_H_
 
 #include <algorithm>
+#include <list>
 #include <mutex>
 #include <queue>
 #include <signal.h>
@@ -78,6 +79,7 @@ struct DABlinGTKOptions {
 	std::string displayed_channels;
 	std::string initial_channel;
 	std::string recordings_path;
+	size_t rec_prebuffer_size_s;
 	bool pcm_output;
 	bool untouched_output;
 	int gain;
@@ -89,6 +91,7 @@ DABlinGTKOptions() :
 	initial_scids(LISTED_SERVICE::scids_none),
 	dab_live_source_type(DABLiveETISource::TYPE_DAB2ETI),
 	recordings_path("/tmp"),
+	rec_prebuffer_size_s(0),
 	pcm_output(false),
 	untouched_output(false),
 	gain(DAB_LIVE_SOURCE_CHANNEL::auto_gain),
@@ -126,6 +129,21 @@ public:
 };
 
 
+// --- RecSample -----------------------------------------------------------------
+struct RecSample {
+	std::vector<uint8_t> data;
+	size_t duration_ms;
+
+	RecSample(const uint8_t* data, size_t len, size_t duration_ms) {
+		this->data.resize(len);
+		memcpy(&this->data[0], data, len);
+		this->duration_ms = duration_ms;
+	}
+};
+
+typedef std::list<RecSample> rec_samples_t;
+
+
 // --- DABlinGTK -----------------------------------------------------------------
 class DABlinGTK : public Gtk::Window, ETISourceObserver, ETIPlayerObserver, FICDecoderObserver, PADDecoderObserver, UntouchedStreamConsumer {
 private:
@@ -144,24 +162,28 @@ private:
 	FICDecoder *fic_decoder;
 	PADDecoder *pad_decoder;
 
-	std::mutex rec_file_mutex;
+	std::mutex rec_mutex;
 	FILE* rec_file;
+	std::string rec_filename;
+	long int rec_duration_ms;
+	rec_samples_t rec_prebuffer;
+	long int rec_prebuffer_filled_ms;
 
 	// ETI data change
 	GTKDispatcherQueue<ETI_PROGRESS> eti_update_progress;
 	void ETIProcessFrame(const uint8_t *data) {eti_player->ProcessFrame(data);}
-	void ETIUpdateProgress(const ETI_PROGRESS progress) {eti_update_progress.PushAndEmit(progress);}
+	void ETIUpdateProgress(const ETI_PROGRESS& progress) {eti_update_progress.PushAndEmit(progress);}
 	void ETIUpdateProgressEmitted();
 
-	GTKDispatcherQueue<std::string> eti_change_format;
-	void ETIChangeFormat(const std::string& format) {eti_change_format.PushAndEmit(format);}
+	GTKDispatcherQueue<AUDIO_SERVICE_FORMAT> eti_change_format;
+	void ETIChangeFormat(const AUDIO_SERVICE_FORMAT& format) {eti_change_format.PushAndEmit(format);}
 	void ETIChangeFormatEmitted();
 
 	void ETIProcessFIC(const uint8_t *data, size_t len) {fic_decoder->Process(data, len);}
 	void ETIResetFIC() {fic_decoder->Reset();}
 	void ETIProcessPAD(const uint8_t *xpad_data, size_t xpad_len, bool exact_xpad_len, const uint8_t* fpad_data) {pad_decoder->Process(xpad_data, xpad_len, exact_xpad_len, fpad_data);}
 
-	void ProcessUntouchedStream(const uint8_t* data, size_t len);
+	void ProcessUntouchedStream(const uint8_t* data, size_t len, size_t duration_ms);
 
 
 	Gtk::Grid top_grid;
@@ -197,7 +219,7 @@ private:
 
 	void InitWidgets();
 	void AddChannels();
-	void AddChannel(dab_channels_t::const_iterator &it, int gain);
+	void AddChannel(const dab_channels_t::value_type& dab_channel, int gain);
 
 	void SetService(const LISTED_SERVICE& service);
 
@@ -213,6 +235,7 @@ private:
 	bool HandleKeyPressEvent(GdkEventKey* key_event);
 	bool HandleConfigureEvent(GdkEventConfigure* configure_event);
 	void TryServiceSwitch(int index);
+	void UpdateRecStatus();
 
 	// FIC data change
 	GTKDispatcherQueue<FIC_ENSEMBLE> fic_change_ensemble;
@@ -223,6 +246,8 @@ private:
 	void FICChangeService(const LISTED_SERVICE& service) {fic_change_service.PushAndEmit(service);}
 	void FICChangeServiceEmitted();
 
+	void FICDiscardedFIB();
+
 	// PAD data change
 	GTKDispatcherQueue<DL_STATE> pad_change_dynamic_label;
 	void PADChangeDynamicLabel(const DL_STATE& dl) {pad_change_dynamic_label.PushAndEmit(dl);}
@@ -234,7 +259,7 @@ private:
 
 	void PADLengthError(size_t announced_xpad_len, size_t xpad_len);
 
-	Glib::ustring DeriveShortLabel(Glib::ustring long_label, uint16_t short_label_mask);
+	static Glib::ustring DeriveShortLabel(Glib::ustring long_label, uint16_t short_label_mask);
 public:
 	DABlinGTK(DABlinGTKOptions options);
 	~DABlinGTK();
