@@ -41,7 +41,7 @@ class ETIPlayerObserver {
 public:
 	virtual ~ETIPlayerObserver() {}
 
-	virtual void ETIChangeFormat(const std::string& /*format*/) {}
+	virtual void ETIChangeFormat(const AUDIO_SERVICE_FORMAT& /*format*/) {}
 	virtual void ETIProcessFIC(const uint8_t* /*data*/, size_t /*len*/) {}
 	virtual void ETIProcessPAD(const uint8_t* /*xpad_data*/, size_t /*xpad_len*/, bool /*exact_xpad_len*/, const uint8_t* /*fpad_data*/) {}
 };
@@ -54,6 +54,7 @@ private:
 	ETIPlayerObserver *observer;
 
 	std::chrono::steady_clock::time_point next_frame_time;
+	uint32_t prev_fsync;
 
 	std::mutex audio_service_mutex;
 	AUDIO_SERVICE audio_service;
@@ -63,12 +64,17 @@ private:
 
 	void DecodeFrame(const uint8_t *eti_frame);
 
-	void FormatChange(const std::string& format);
+	void FormatChange(const AUDIO_SERVICE_FORMAT& format);
 	void StartAudio(int samplerate, int channels, bool float32) {if(out) out->StartAudio(samplerate, channels, float32);}
 	void PutAudio(const uint8_t *data, size_t len) {if(out) out->PutAudio(data, len);}
+
 	void ProcessFIC(const uint8_t *data, size_t len);
 	void ProcessPAD(const uint8_t *xpad_data, size_t xpad_len, bool exact_xpad_len, const uint8_t *fpad_data);
 	void ProcessUntouchedStream(const uint8_t* data, size_t len);
+
+	void AudioError(const std::string& hint);
+	void AudioWarning(const std::string& hint);
+	void FECInfo(int total_corr_count, bool uncorr_errors);
 public:
 	ETIPlayer(bool pcm_output, bool untouched_output, ETIPlayerObserver *observer);
 	~ETIPlayer();
