@@ -29,6 +29,8 @@
 #include <unistd.h>
 #include <fcntl.h>
 
+#include "tools.h"
+
 
 struct ETI_PROGRESS {
 	double value;
@@ -59,7 +61,7 @@ public:
 	virtual ~ETISourceObserver() {}
 
 	virtual void ETIProcessFrame(const uint8_t* /*data*/) {}
-	virtual void ETIUpdateProgress(const ETI_PROGRESS /*progress*/) {}
+	virtual void ETIUpdateProgress(const ETI_PROGRESS& /*progress*/) {}
 };
 
 
@@ -82,8 +84,6 @@ protected:
 	bool UpdateTotalFrames();
 	virtual void Init() {}
 	virtual void PrintSource();
-
-	static std::string FramecountToTimecode(size_t value);
 public:
 	ETISource(std::string filename, ETISourceObserver *observer);
 	virtual ~ETISource();
@@ -118,7 +118,7 @@ class DAB2ETIETISource : public DABLiveETISource {
 protected:
 	std::string GetParams();
 public:
-	DAB2ETIETISource(std::string binary, DAB_LIVE_SOURCE_CHANNEL channel, ETISourceObserver *observer) : DABLiveETISource(binary, channel, observer, "dab2eti") {}
+	DAB2ETIETISource(std::string binary, DAB_LIVE_SOURCE_CHANNEL channel, ETISourceObserver *observer) : DABLiveETISource(binary, channel, observer, TYPE_DAB2ETI) {}
 };
 
 
@@ -127,7 +127,7 @@ class EtiCmdlineETISource : public DABLiveETISource {
 protected:
 	std::string GetParams();
 public:
-	EtiCmdlineETISource(std::string binary, DAB_LIVE_SOURCE_CHANNEL channel, ETISourceObserver *observer) : DABLiveETISource(binary, channel, observer, "eti-cmdline") {}
+	EtiCmdlineETISource(std::string binary, DAB_LIVE_SOURCE_CHANNEL channel, ETISourceObserver *observer) : DABLiveETISource(binary, channel, observer, TYPE_ETI_CMDLINE) {}
 };
 
 #endif /* ETI_SOURCE_H_ */
