@@ -182,6 +182,8 @@ public:
 
 	virtual void FICChangeEnsemble(const FIC_ENSEMBLE& /*ensemble*/) {}
 	virtual void FICChangeService(const LISTED_SERVICE& /*service*/) {}
+
+	virtual void FICDiscardedFIB() {}
 };
 
 
@@ -236,7 +238,7 @@ public:
 	void Reset();
 
 	static std::string ConvertTextToUTF8(const uint8_t *data, size_t len, int charset, bool mot, std::string* charset_name);
-	static std::string ConvertLabelToUTF8(const FIC_LABEL& label);
+	static std::string ConvertLabelToUTF8(const FIC_LABEL& label, std::string* charset_name);
 	static std::string ConvertLanguageToString(const int value);
 };
 
